// ------------------------------------------------------------
// Machinery project - math library
// ------------------------------------------------------------
// (c) 2000 by Sascha 'EvilOne' Salevsky
// E-mail: Sascha.Salevsky@Ibykus.de
// ------------------------------------------------------------

#ifndef _mathematics_h_
#error "Dont include Plane directly!!! Use Mathematics.h instead."
#endif

inline Plane::Plane()
{}

// copy by hand avoids call to Vector3::operator=(...)
inline Plane::Plane(const Vector3& _n, float _d)
{
	a = _n.x; b = _n.y; c = _n.z; d = _d;
}

inline Plane::Plane(float _a, float _b, float _c, float _d)
{
	a = _a; b = _b; c = _c; d = _d;
}

inline Plane::Plane(float* other)
{
	a = other[0]; b = other[1]; c = other[2]; d = other[3];
}

inline Plane::Plane(const Plane& other)
{
	a = other.a; b = other.b; c = other.c; d = other.d;
}

// Assignment

inline Plane& Plane::operator=(const Plane& other)
{
	a = other.a; b = other.b; c = other.c; d = other.d;
	return *this;
}

inline void Plane::Assign(const Vector3& _n, float _d)
{
	a = _n.x; b = _n.y; c = _n.z; d = _d;
}

inline void Plane::Assign(float _a, float _b, float _c, float _d)
{
	a = _a; b = _b; c = _c; d = _d;
}

inline void Plane::Assign(float* other)
{
	a = other[0]; b = other[1]; c = other[2]; d = other[3];
}

inline void Plane::Assign(const Plane& other)
{
	a = other.a; b = other.b; c = other.c; d = other.d;
}

// Functions

inline float Plane::Distance(const Vector3& v) const
{
	return n.Dot(v) - d;
}

inline void Plane::Transform(const Matrix4& mat)
{
  const float* m = mat.M;
  float fx = a;
  float fy = b;
  float fz = c;
  float fw = d;
  a = m[0] * fx + m[4] * fy + m[8] * fz + m[12] * fw;
  b = m[1] * fx + m[5] * fy + m[9] * fz + m[13] * fw;
  c = m[2] * fx + m[6] * fy + m[10] * fz + m[14] * fw;
  d = m[3] * fx + m[7] * fy + m[11] * fz + m[15] * fw;
}

inline void Plane::Transform(const Matrix4& mat, const Plane& other)
{
  const float* m = mat.M;
  float fx = other.a;
  float fy = other.b;
  float fz = other.c;
  float fw = other.d;
  a = m[0] * fx + m[4] * fy + m[8] * fz + m[12] * fw;
  b = m[1] * fx + m[5] * fy + m[9] * fz + m[13] * fw;
  c = m[2] * fx + m[6] * fy + m[10] * fz + m[14] * fw;
  d = m[3] * fx + m[7] * fy + m[11] * fz + m[15] * fw;
}
