// ------------------------------------------------------------
// Machinery project - math library
// ------------------------------------------------------------
// (c) 2000 by Sascha 'EvilOne' Salevsky
// E-mail: Sascha.Salevsky@Ibykus.de
// ------------------------------------------------------------

#ifndef _mathematics_h_
#error "Dont include Vector3 directly!!! Use Mathematics.h instead."
#endif

// Constructors

inline Vector3::Vector3()
{}

inline Vector3::Vector3(float _x, float _y, float _z)
{
	x = _x; y = _y; z = _z;
}

inline Vector3::Vector3(Vector3& other)
{
	x = other.x; y = other.y; z = other.z;
}

inline Vector3::Vector3(const float* other)
{
	x = other[0]; y = other[1]; z = other[2];
}

// assignemt

inline Vector3& Vector3::operator=(const Vector3& other)
{
	x = other.x; y = other.y; z = other.z;
	return *this;
}

inline void Vector3::Assign(float _x, float _y, float _z)
{
	x = _x; y = _y; z = _z;
}

inline void Vector3::Assign(const Vector3& other)
{
	x = other.x; y = other.y; z = other.z;
}

inline void Vector3::Assign(float* other)
{
	x = other[0]; y = other[1]; z = other[2];
}

// Swap

inline void Vector3::Swap(Vector3& other)
{
	float _x = x, _y = y, _z = z;
	x = other.x; y = other.y; z = other.z;
	other.x = _x; other.y = _y; other.z = _z;
}

// Casting

inline Vector3::operator float*()
{
	return &x;
}

inline Vector3::operator const float*() const
{
	return &x;
}

// base functions

inline void Vector3::Add(const Vector3& other)
{
	x += other.x; y += other.y; z += other.z;
}

inline void Vector3::Add(const Vector3& v1, const Vector3& v2)
{
	x = v1.x + v2.x; y = v1.y + v2.y; z = v1.z + v2.z;
}

inline void Vector3::Sub(const Vector3& other)
{
	x -= other.x; y -= other.y; z -= other.z;
}

inline void Vector3::Sub(const Vector3& v1, const Vector3& v2)
{
	x = v1.x - v2.x; y = v1.y - v2.y; z = v1.z - v2.z;
}

inline void Vector3::Mul(float f)
{
	x *= f; y *= f; z *= f;
}

inline void Vector3::Mul(const Vector3& other, float f)
{
	x = other.x * f; y = other.y * f; z = other.z * f;
}

inline void Vector3::Div(float f)
{
	f = 1.0f / f;
	x *= f; y *= f; z *= f;
}

inline void Vector3::Div(const Vector3& other, float f)
{
	f = 1.0f / f;
	x = other.x * f; y = other.y * f; z = other.z * f;
}

inline void Vector3::Invert()
{
	x = -x; x = -y; z = -z;
}

inline void Vector3::Invert(const Vector3& other)
{
	x = -other.x; x = -other.y; z = -other.z;
}

// Linear combination

inline void Vector3::Combine(const Vector3& other, float s)
{
	x += other.x * s; y += other.y * s; z += other.z * s;
}

inline void Vector3::Combine(float s, const Vector3& other, float t)
{
	x = x * s + other.x * t;
	y = y * s + other.y * t;
	z = z * s + other.z * t;
}

inline void Vector3::Combine(const Vector3& v1, float s, const Vector3& v2, float t)
{
	x = v1.x * s + v2.x * t;
	y = v1.y * s + v2.y * t;
	z = v1.z * s + v2.z * t;
}

// EvilOne's beloved overloads

inline Vector3& Vector3::operator+=(const Vector3& other)
{
	x += other.x; y += other.y; z += other.z;
	return *this;
}

inline Vector3& Vector3::operator-=(const Vector3& other)
{
	x -= other.x; y -= other.y; z -= other.z;
	return *this;
}

inline Vector3& Vector3::operator*=(float f)
{
	x *= f; y *= f; z *= f;
	return *this;
}

inline Vector3& Vector3::operator-=(float f)
{
	x -= f; y -= f; z -= f;
	return *this;
}

inline Vector3& Vector3::operator+=(float f)
{
	x += f; y += f; z += f;
	return *this;
}

inline Vector3& Vector3::operator/=(float f)
{
	f = 1.0f / f;
	x *= f; y *= f; z *= f;
	return *this;
}

inline Vector3& Vector3::operator-()
{
	x = -x; y = -y; z = -z;
	return *this;
}

inline Vector3 Vector3::operator+(const Vector3& other) const
{
	return Vector3(x + other.x, y + other.y, z + other.z);
}

inline Vector3 Vector3::operator-(const Vector3& other) const
{
	return Vector3(x - other.x, y - other.y, z - other.z);
}

inline Vector3 Vector3::operator+(float f) const
{
	return Vector3(x + f, y + f, z + f);
}

inline Vector3 Vector3::operator-(float f) const
{
	return Vector3(x - f, y - f, z - f);
}

inline Vector3 Vector3::operator*(float f) const
{
	return Vector3(x * f, y * f, z * f);
}

inline Vector3 Vector3::operator/(float f) const
{
	f = 1.0f / f;
	return Vector3(x * f, y * f, z * f);
}

// Linear Algebra

inline float Vector3::Dot(const Vector3& v) const
{
	return x * v.x + y * v.y + z * v.z;
}

inline void Vector3::Cross(const Vector3& v)
{
	float _x = y * v.z - z * v.y;
	float _y = z * v.x - x * v.z;
	float _z = x * v.y - y * v.x;
	x = _x; y = _y; z = _z;
}

inline void Vector3::Cross(const Vector3& v1, const Vector3& v2)
{
	float _x = v1.y * v2.z - v1.z * v2.y;
	float _y = v1.z * v2.x - v1.x * v2.z;
	float _z = v1.x * v2.y - v1.y * v2.x;
	x = _x; y = _y; z = _z;
}

inline float Vector3::Length() const
{
	return (float)sqrt(x * x + y * y + z * z);
}

inline float Vector3::LengthSqr() const
{
	return x * x + y * y + z * z;
}

inline float Vector3::Distance(const Vector3& other) const
{
	Vector3 tmp = other - *this;
	return tmp.Length();
}

inline float Vector3::DistanceSqr(const Vector3& other) const
{
	Vector3 tmp = other - *this;
	return tmp.LengthSqr();
}

inline void Vector3::Normalize()
{
	float f = 1.0f / Length();
	x = x * f; y = y * f; z = z * f;
}

inline void Vector3::Normalize(const Vector3& other)
{
	float f = 1.0f / other.Length();
	x = other.x * f; y = other.y * f; z = other.z * f;
}


// Other usefull funtions

inline void Vector3::Clamp(float min, float max)
{
	x = ::Clamp(x, min, max);
	y = ::Clamp(y, min, max);
	z = ::Clamp(z, min, max);
}

inline void Vector3::Lerp(const Vector3& v1, const Vector3& v2, float s)
{
	x = ::Lerp(s, v1.x, v2.x);
	y = ::Lerp(s, v1.y, v2.y);
	z = ::Lerp(s, v1.z, v2.z);
}

// Operations with a plane

inline void Vector3::Intersect(const Vector3& start, float s, const Vector3& end, float e)
{
	float ws = s / (e - s);
	float we = e / (e - s);
	x = start.x * ws + end.x * we;
	y = start.y * ws + end.y * we;
	z = start.z * ws + end.z * we;
}

inline void Vector3::Intersect(const Plane& p, const Vector3& start, const Vector3& end)
{
	float s = p.Distance(start);
	float e = p.Distance(end);
	Intersect(start, s, end, e);
}

inline void Vector3::Reflect(const Plane& p, const Vector3& v)
{
	Vector3 tmp = p.n * 2 * v.Dot(p.n) - v;
	x = tmp.x; y = tmp.y; z = tmp.z;
}

// Operations with a matrix

inline void Vector3::Transform(const Matrix4& mat)
{
	const float* m = mat.M;
	float fx = x; float fy = y; float fz = z;
	x = m[0] * fx + m[4] * fy + m[8] * fz + m[12];  // w assumed to be 1.0F
	y = m[1] * fx + m[5] * fy + m[9] * fz + m[13];
	z = m[2] * fx + m[6] * fy + m[10] * fz + m[14];
}

inline void Vector3::Transform(const Matrix4& mat, const Vector3& other)
{
	const float* m = mat.M;
	float fx = other.x; float fy = other.y; float fz = other.z;
	x = m[0] * fx + m[4] * fy + m[8] * fz + m[12];  // w assumed to be 1.0F
	y = m[1] * fx + m[5] * fy + m[9] * fz + m[13];
	z = m[2] * fx + m[6] * fy + m[10] * fz + m[14];
}

inline void Vector3::TransformTranspose(const Matrix4& mat)
{
	const float* m = mat.M;
	float fx = x; float fy = y; float fz = z;
	x = m[0] * fx + m[1] * fy + m[2] * fz + m[3];  // w assumed to be 1.0F
	y = m[4] * fx + m[5] * fy + m[6] * fz + m[7];
	z = m[8] * fx + m[9] * fy + m[10] * fz + m[11];
}

inline void Vector3::TransformTranspose(const Matrix4& mat, const Vector3& other)
{
	const float* m = mat.M;
	float fx = other.x; float fy = other.y; float fz = other.z;
	x = m[0] * fx + m[1] * fy + m[2] * fz + m[3];  // w assumed to be 1.0F
	y = m[4] * fx + m[5] * fy + m[6] * fz + m[7];
	z = m[8] * fx + m[9] * fy + m[10] * fz + m[11];
}

